import { mergeConfig, build as build$1 } from "vite";
import { builtinModules } from "node:module";
function defineConfig(config) {
  return config;
}
function resolveViteConfig(option) {
  const defaultConfig = {
    configFile: false,
    publicDir: false,
    build: {
      lib: option.entry && {
        entry: option.entry,
        formats: ["cjs"],
        fileName: () => "[name].js"
      },
      outDir: "dist-electron",
      emptyOutDir: false
    },
    resolve: {
      mainFields: ["module", "jsnext:main", "jsnext"]
    }
  };
  return mergeConfig(defaultConfig, (option == null ? void 0 : option.vite) || {});
}
function withExternalBuiltins(config) {
  var _a;
  const builtins = builtinModules.filter((e) => !e.startsWith("_"));
  builtins.push("electron", ...builtins.map((m) => `node:${m}`));
  config.build ?? (config.build = {});
  (_a = config.build).rollupOptions ?? (_a.rollupOptions = {});
  let external = config.build.rollupOptions.external;
  if (Array.isArray(external) || typeof external === "string" || external instanceof RegExp) {
    external = builtins.concat(external);
  } else if (typeof external === "function") {
    const original = external;
    external = function(source, importer, isResolved) {
      if (builtins.includes(source)) {
        return true;
      }
      return original(source, importer, isResolved);
    };
  } else {
    external = builtins;
  }
  config.build.rollupOptions.external = external;
  return config;
}
function resolveHostname(hostname) {
  const loopbackHosts = /* @__PURE__ */ new Set([
    "localhost",
    "127.0.0.1",
    "::1",
    "0000:0000:0000:0000:0000:0000:0000:0001"
  ]);
  const wildcardHosts = /* @__PURE__ */ new Set([
    "0.0.0.0",
    "::",
    "0000:0000:0000:0000:0000:0000:0000:0000"
  ]);
  return loopbackHosts.has(hostname) || wildcardHosts.has(hostname) ? "localhost" : hostname;
}
function resolveServerUrl(server) {
  const addressInfo = server.httpServer.address();
  const isAddressInfo = (x) => x == null ? void 0 : x.address;
  if (isAddressInfo(addressInfo)) {
    const { address, port } = addressInfo;
    const hostname = resolveHostname(address);
    const options = server.config.server;
    const protocol = options.https ? "https" : "http";
    const devBase = server.config.base;
    const path = typeof options.open === "string" ? options.open : devBase;
    const url = path.startsWith("http") ? path : `${protocol}://${hostname}:${port}${path}`;
    return url;
  }
}
function build(config) {
  return build$1(withExternalBuiltins(resolveViteConfig(config)));
}
function electron(config) {
  const configArray = Array.isArray(config) ? config : [config];
  let mode;
  return [
    {
      name: "vite-plugin-electron",
      apply: "serve",
      configureServer(server) {
        var _a;
        (_a = server.httpServer) == null ? void 0 : _a.once("listening", () => {
          var _a2, _b, _c, _d;
          Object.assign(process.env, {
            VITE_DEV_SERVER_URL: resolveServerUrl(server)
          });
          for (const config2 of configArray) {
            config2.vite ?? (config2.vite = {});
            (_a2 = config2.vite).mode ?? (_a2.mode = server.config.mode);
            (_b = config2.vite).build ?? (_b.build = {});
            (_c = config2.vite.build).watch ?? (_c.watch = {});
            (_d = config2.vite).plugins ?? (_d.plugins = []);
            config2.vite.plugins.push({
              name: ":startup",
              closeBundle() {
                if (config2.onstart) {
                  config2.onstart.call(this, {
                    startup,
                    reload() {
                      server.ws.send({ type: "full-reload" });
                    }
                  });
                } else {
                  startup();
                }
              }
            });
            build(config2);
          }
        });
      }
    },
    {
      name: "vite-plugin-electron",
      apply: "build",
      config(config2, env) {
        config2.base ?? (config2.base = "./");
        mode = env.mode;
      },
      async closeBundle() {
        var _a;
        for (const config2 of configArray) {
          config2.vite ?? (config2.vite = {});
          (_a = config2.vite).mode ?? (_a.mode = mode);
          await build(config2);
        }
      }
    }
  ];
}
async function startup(argv = [".", "--no-sandbox"]) {
  const { spawn } = await import("node:child_process");
  const electron2 = await import("electron");
  const electronPath = electron2.default ?? electron2;
  startup.exit();
  process.electronApp = spawn(electronPath, argv, { stdio: "inherit" });
  process.electronApp.once("exit", process.exit);
  if (!startup.hookProcessExit) {
    startup.hookProcessExit = true;
    process.once("exit", startup.exit);
  }
}
startup.hookProcessExit = false;
startup.exit = () => {
  if (process.electronApp) {
    process.electronApp.removeAllListeners();
    import("tree-kill").then(({ default: kill }) => kill(process.electronApp.pid));
  }
};
export {
  build,
  electron as default,
  defineConfig,
  resolveViteConfig,
  startup,
  withExternalBuiltins
};
