import { type InlineConfig, type ViteDevServer } from 'vite';
export interface Configuration {
    /**
     * Shortcut of `build.lib.entry`
     */
    entry?: import('vite').LibraryOptions['entry'];
    vite?: import('vite').InlineConfig;
    /**
     * Triggered when Vite is built every time -- `vite serve` command only.
     *
     * If this `onstart` is passed, Electron App will not start automatically.
     * However, you can start Electroo App via `startup` function.
     */
    onstart?: (this: import('rollup').PluginContext, options: {
        /**
         * Electron App startup function.
         * It will mount the Electron App child-process to `process.electronApp`.
         * @param argv default value `['.', '--no-sandbox']`
         */
        startup: (argv?: string[]) => Promise<void>;
        /** Reload Electron-Renderer */
        reload: () => void;
    }) => void | Promise<void>;
}
export declare function defineConfig(config: Configuration): Configuration;
/** Resolve the default Vite's `InlineConfig` for build Electron-Main */
export declare function resolveViteConfig(option: Configuration): InlineConfig;
export declare function withExternalBuiltins(config: InlineConfig): InlineConfig;
/**
 * @see https://github.com/vitejs/vite/blob/v4.0.1/packages/vite/src/node/constants.ts#L137-L147
 */
export declare function resolveHostname(hostname: string): string;
export declare function resolveServerUrl(server: ViteDevServer): string | void;
